/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.UnknownNamespaceException;

import com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery.XmlPathDiscovery;

import java.io.IOException;

import java.util.Arrays;
import java.util.List;
import java.util.ListIterator;

import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.BodyContent;
import javax.servlet.jsp.tagext.BodyTagSupport;


/**
 * <p>
 * The main tag class that application will use. This tag will render the current node to the root node (or vise-versa)
 * using a path separator and specific styles. The application can setup the separator and the various styles either
 * by specifying default values in the configuration manager or by specifying attributes on the string. The object
 * factory will be used to discover the node and the path to the root. The node discovery is optional. If the node
 * discovery is not specified, it is assumed a node discovery tag was used on the page to define the current node. The
 * body of the tag will be used to format the various trails (using variables defined by this class).
 * </p>
 *
 * <p>
 * <b>Changes in Version 2.0 :</b>
 *
 * <ol>
 * <li>
 * This class now inherits from <code>BodyTagSupport</code>.
 * </li>
 * <li>
 * Changed all <code>NullPointerExceptions</code> to <code>IllegalArgumentException</code> and changed all string
 * setters to save a <code>null</code> when it's an empty string expect for the pathSeparator field.
 * </li>
 * <li>
 * Eliminated the SITE_MAP_ATTRIBUTE_NAME
 * </li>
 * <li>
 * Eliminated dataSource and associated getter/setter
 * </li>
 * <li>
 * Eliminated siteDataSource and associated getter/setter
 * </li>
 * <li>
 * Eliminated parserClassName and associated getter/setter
 * </li>
 * <li>
 * Eliminated the private variable &quot;out&quot;
 * </li>
 * <li>
 * Eliminated the parseSiteDataSource private method
 * </li>
 * <li>
 * Eliminated the printCurrentRootNode private method
 * </li>
 * <li>
 * Eliminated the printCurrentNode private method
 * </li>
 * <li>
 * Eliminated the printRootNode private method
 * </li>
 * <li>
 * Eliminated the printNode private method
 * </li>
 * <li>
 * Eliminated the printN private method
 * </li>
 * <li>
 * Added, documented and added out argument to printStyleSettings private method
 * </li>
 * <li>
 * Added, documented and added out argument to printStyle private method
 * </li>
 * <li>
 * Added, documented and added out argument to printSeparator method
 * </li>
 * <li>
 * Added, documented and added out argument to getSilentErrors/setSilentErrors method
 * </li>
 * <li>
 * Added constructor
 * </li>
 * <li>
 * Changed doStartTag fully
 * </li>
 * <li>
 * Added doInitBody method
 * </li>
 * <li>
 * Added doAfterBody method
 * </li>
 * <li>
 * Added doEndTag method
 * </li>
 * <li>
 * Added iterator private variables
 * </li>
 * <li>
 * Added title private variable and associated getter/setter methods
 * </li>
 * <li>
 * Added pathOverride private variable and associated getter/setter methods
 * </li>
 * </ol>
 * </p>
 *
 * <p>
 * <strong>Thread Safety:</strong> This class is thread safe in the context of the JSP rendering process.
 * </p>
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 *
 * @since 1.0
 */
public class BreadCrumbTrailTag extends BodyTagSupport {
    /**
     * <p>
     * Represents the page scope attribute name that the current node would be found under. This constant will be used
     * by a <code>NodeDiscovery</code> instance to store the current node in the attribute map and by this class to
     * retrieve.
     * </p>
     *
     * @since 2.0
     */
    public static final String BREADCRUMB_CURRENT_NODE = "breadCrumbCurrentNode";

    /**
     * <p>
     * Represents the nested variable name that the current node to format would be found under. This constant will be
     * used by this class to define a variable available to a <code>NodeFormatter</code> to format. Note: this is the
     * page scope attribute name that will be made available as a variable via the
     * <code>BreadCrumbTrailTagExtraInfo</code> class.
     * </p>
     *
     * @since 2.0
     */
    public static final String BREADCRUMB_FORMAT_NODE = "breadCrumbFormatNode";

    /**
     * <p>
     * Represents the nested variable name that the current node's style that will be used to format. This constant
     * will be used by this class to define a variable available to a <code>NodeFormatter</code> to format. Note: this
     * is the page scope attribute name that will be made available as a variable via the
     * <code>BreadCrumbTrailTagExtraInfo</code> class.
     * </p>
     *
     * @since 2.0
     */
    public static final String BREADCRUMB_FORMAT_STYLE_CLASS = "breadCrumbFormatStyleClass";

    /**
     * <p>
     * Represents a pathDirection constant for formatting the path from the root (on the left) to the current node (on
     * the right).
     * </p>
     */
    public static final int ROOT_TO_CURRENT = 0;

    /**
     * <p>
     * Represents a pathDirection constant for formatting the path from the current node (on the left) to the root node
     * (on the right).
     * </p>
     */
    public static final int CURRENT_TO_ROOT = 1;

    /**
     * <p>
     * Represents the default namespace used to load the config values from config manager.
     * </p>
     *
     * @since 2.0
     */
    private static final String DEFAULT_NAMESPACE = BreadCrumbTrailTag.class.getPackage().getName();

    /**
     * <p>
     * Represents the default namespace used to load the config values from config manager for object factory.
     * </p>
     *
     * @since 2.0
     */
    private static final String OBJECT_FACTORY_NAMESPACE = BreadCrumbTrailTag.class.getName();

    /**
     * <p>
     * Represents the property name to load the config value of pathSeparator.
     * </p>
     *
     * @since 2.0
     */
    private static final String PATH_SEPARATOR_PROPERTY = "pathSeparator";

    /**
     * <p>
     * Represents the property name to load the config value of pathSeparatorStyle.
     * </p>
     *
     * @since 2.0
     */
    private static final String PATH_SEPARATOR_STYLE_PROPERTY = "pathSeparatorStyle";

    /**
     * <p>
     * Represents the property name to load the config value of nodeStyle.
     * </p>
     *
     * @since 2.0
     */
    private static final String NODE_STYLE_PROPERTY = "nodeStyle";

    /**
     * <p>
     * Represents the property name to load the config value of rootNodeStyle.
     * </p>
     *
     * @since 2.0
     */
    private static final String ROOT_NODE_STYLE_PROPERTY = "rootNodeStyle";

    /**
     * <p>
     * Represents the property name to load the config value of currentNodeStyle.
     * </p>
     *
     * @since 2.0
     */
    private static final String CURRENT_NODE_STYLE_PROPERTY = "currentNodeStyle";

    /**
     * <p>
     * Represents the property name to load the config value of mouseOverStyle.
     * </p>
     *
     * @since 2.0
     */
    private static final String MOUSE_OVER_STYLE_PROPERTY = "mouseOverStyle";

    /**
     * <p>
     * Represents the property name to load the config value of pathDirection.
     * </p>
     *
     * @since 2.0
     */
    private static final String PATH_DIRECTION_PROPERTY = "pathDirection";

    /**
     * <p>
     * Represents the property name to load the config value of silentErrors.
     * </p>
     *
     * @since 2.0
     */
    private static final String SILENT_ERRORS_PROPERTY = "silentErrors";

    /**
     * <p>
     * Represents the separator that will be drawn between nodes in a path. This variable is set by the constructor, is
     * mutable and will never be <code>null</code> but can be an empty (trim'd) string. Its default value is ':'.
     * </p>
     */
    private String pathSeparator = ":";

    /**
     * <p>
     * Represents the separator CSS style that will be used to draw the separator. This is the actual CSS style that
     * will be rendered to the underlying text. This variable is set by the constructor, is mutable and can be
     * <code>null</code> or an empty (trim'd) string to specify no specific style (ie use the style that is currently
     * in context).
     * </p>
     */
    private String pathSeparatorStyle = null;

    /**
     * <p>
     * Represents the node (non-root and non-current) CSS style that will be used to draw the node's title. This is the
     * actual CSS style that will be rendered to the underlying link. This variable is set by the constructor, is
     * mutable and can be <code>null</code> or an empty (trim'd) string to specify no specific style (ie use the style
     * that is currently in context).
     * </p>
     */
    private String nodeStyle = null;

    /**
     * <p>
     * Represents the current (only) node CSS style that will be used to draw the node's title. This is the actual CSS
     * style that will be rendered to the underlying link. This variable is set by the constructor, is mutable and can
     * be <code>null</code> or an empty (trim'd) string to specify no specific style (ie use the style that is
     * currently in context).
     * </p>
     */
    private String currentNodeStyle = null;

    /**
     * <p>
     * Represents the root (only) node CSS style that will be used to draw the node's title. This is the actual CSS
     * style that will be rendered to the underlying link. This variable is set by the constructor, is mutable and can
     * be <code>null</code> or an empty (trim'd) string to specify no specific style (ie use the style that is
     * currently in context).
     * </p>
     */
    private String rootNodeStyle = null;

    /**
     * <p>
     * Represents the mouse over node CSS style that will be used to draw the node's title. This is the actual CSS
     * style that will be rendered to the underlying link. This variable is set by the constructor, is mutable and can
     * be <code>null</code> or an empty (trim'd) string to specify no specific style (ie use the style that is
     * currently in context).
     * </p>
     */
    private String mouseOverStyle = null;

    /**
     * <p>
     * Represents whether any exceptions should be silently ignore or not (<code>true</code> to silently ignore,
     * <code>false</code> to let exception pass through to Jsp engine). This variable is set by the constructor and is
     * mutatable. Its default value is <code>false</code>.
     * </p>
     */
    private boolean silentErrors = false;

    /**
     * <p>
     * Represents the direction of the path. The current allowable values are either ROOT_TO_CURRENT (specifying root
     * [on the left] to current [on the right]) or CURRENT_TO_ROOT (specifying current [on the left] to root [on the
     * right]). This variable is set by the constructor, is mutable and can only be the above values. Its default
     * value is root to current.
     * </p>
     */
    private int pathDirection = ROOT_TO_CURRENT;

    /**
     * <p>
     * Represents the path override for this specific tag. This variable is set initially to <code>null</code> and is
     * mutatable. If <code>null</code>, there is no path override available and normal path discovery should be used.
     * If non-null, a <code>XmlPathDiscovery</code> class will be constructed with this path and used to return the
     * path specified in this variable. Please see <code>XmlPathDiscovery</code> class for details on it's formatting.
     * </p>
     *
     * @since 2.0
     */
    private String pathOverride = null;

    /**
     * <p>
     * Represents title that will be used if the node is being discovered by this tag (ie no other tag has already
     * discovered the node and set the BREADCRUMB_CURRENT_NODE attribute) - otherwise it is ignored. This variable is
     * set initially to <code>null</code>, is mutatable and never will be an empty string but can be
     * <code>null</code>. A <code>null</code> indicates that the title should default to the title specified in the
     * <code>PathDiscovery</code> data source.
     * </p>
     *
     * @since 2.0
     */
    private String title = null;

    /**
     * <p>
     * Represents the iterator over the current node path. This iterator will initially be <code>null</code>, be
     * mutatable by the doStartTag and will be non-null during the body processing. The iterator will be created in
     * the doStartTag from the path discovered by the <code>PathDiscovery</code> implementation. The node formatter
     * (or body if no node formatter specified) will then be called, for each node in this itertor. This variable will
     * only be non-null if a node formatter is not specified (ie that we need the body to format each node).
     * </p>
     *
     * @since 2.0
     */
    private ListIterator iterator = null;

    /**
     * <p>
     * Creates a new instance of <code>BreadCrumbTrailTag</code> class. All the config values will be loaded from the
     * config manager. If the config value is missing or the configuration exception occurs, the default config value
     * will be used instead.
     * </p>
     */
    public BreadCrumbTrailTag() {
        // get all the property values
        this.pathSeparator = getStringPropertyValue(PATH_SEPARATOR_PROPERTY, ":");
        this.pathSeparatorStyle = Helper.convertEmptyString(
                getStringPropertyValue(PATH_SEPARATOR_STYLE_PROPERTY, null));
        this.nodeStyle = Helper.convertEmptyString(getStringPropertyValue(NODE_STYLE_PROPERTY, null));
        this.rootNodeStyle = Helper.convertEmptyString(getStringPropertyValue(ROOT_NODE_STYLE_PROPERTY, null));
        this.currentNodeStyle = Helper.convertEmptyString(getStringPropertyValue(CURRENT_NODE_STYLE_PROPERTY, null));
        this.mouseOverStyle = Helper.convertEmptyString(getStringPropertyValue(MOUSE_OVER_STYLE_PROPERTY, null));
        this.pathDirection = getIntPropertyValue(PATH_DIRECTION_PROPERTY, ROOT_TO_CURRENT);

        if ((this.pathDirection != ROOT_TO_CURRENT) && (pathDirection != CURRENT_TO_ROOT)) {
            this.pathDirection = ROOT_TO_CURRENT;
        }

        this.silentErrors = getBooleanPropertyValue(SILENT_ERRORS_PROPERTY, false);
    }

    /**
     * <p>
     * Gets the direction of the path.
     * </p>
     *
     * @return the direction of the path. - either ROOT_TO_CURRENT or CURRENT_TO_ROOT.
     *
     * @see BreadCrumbTrailTag#pathDirection
     */
    public int getPathDirection() {
        return this.pathDirection;
    }

    /**
     * <p>
     * Sets the direction of the path.
     * </p>
     *
     * @param pathDirection the direction of the path.
     *
     * @throws IllegalArgumentException if pathDirection is not ROOT_TO_CURRENT or CURRENT_TO_ROOT.
     *
     * @see BreadCrumbTrailTag#pathDirection
     */
    public void setPathDirection(int pathDirection) {
        if ((pathDirection != ROOT_TO_CURRENT) && (pathDirection != CURRENT_TO_ROOT)) {
            throw new IllegalArgumentException("The pathDirection is neither ROOT_TO_CURRENT: " + ROOT_TO_CURRENT
                + " nor CURRENT_TO_ROOT: " + CURRENT_TO_ROOT + ".");
        }

        this.pathDirection = pathDirection;
    }

    /**
     * <p>
     * Gets the separator that will be drawn between nodes in a path.
     * </p>
     *
     * @return A non-null, possibly empty (trim'd) string representing the separator that will be drawn between nodes
     *         in a path.
     *
     * @see BreadCrumbTrailTag#pathSeparator
     */
    public String getPathSeparator() {
        return pathSeparator;
    }

    /**
     * <p>
     * Sets the separator that will be drawn between nodes in a path.
     * </p>
     *
     * @param pathSeparator A non-null, possibly empty (trim'd) string representing the separator that will be drawn
     *        between nodes in a path.
     *
     * @throws IllegalArgumentException if the pathSeparator is <code>null</code>.
     *
     * @see BreadCrumbTrailTag#pathSeparator
     */
    public void setPathSeparator(String pathSeparator) {
        Helper.validateNotNull(pathSeparator, "pathSeparator");
        this.pathSeparator = pathSeparator;
    }

    /**
     * <p>
     * Gets the separator CSS style that will be used to draw the separator.
     * </p>
     *
     * @return A possibly <code>null</code>, never empty (trim'd) string representing the separator CSS style that will
     *         be used to draw the separator.
     *
     * @see BreadCrumbTrailTag#pathSeparatorStyle
     */
    public String getPathSeparatorStyle() {
        return pathSeparatorStyle;
    }

    /**
     * <p>
     * Sets the separator CSS style that will be used to draw the separator.
     * </p>
     *
     * @param pathSeparatorStyle A possibly <code>null</code>, possibly empty (trim'd) string representing the
     *        separator CSS style that will be used to draw the separator. If the value is an empty string, it will be
     *        regarded as <code>null</code>.
     *
     * @see BreadCrumbTrailTag#pathSeparatorStyle
     */
    public void setPathSeparatorStyle(String pathSeparatorStyle) {
        this.pathSeparatorStyle = Helper.convertEmptyString(pathSeparatorStyle);
    }

    /**
     * <p>
     * Gets the node (non-root and non-current) CSS style that will be used to draw the node's title.
     * </p>
     *
     * @return A possibly <code>null</code>, never empty (trim'd) string representing the node (non-root and
     *         non-current) CSS style that will be used to draw the node's title.
     *
     * @see BreadCrumbTrailTag#nodeStyle
     */
    public String getNodeStyle() {
        return nodeStyle;
    }

    /**
     * <p>
     * Sets the node (non-root and non-current) CSS style that will be used to draw the node's title.
     * </p>
     *
     * @param nodeStyle A possibly <code>null</code>, possibly empty (trim'd) string representing the node (non-root
     *        and non-current) CSS style that will be used to draw the node's title. If the value is an empty string,
     *        it will be regarded as <code>null</code>.
     *
     * @see BreadCrumbTrailTag#nodeStyle
     */
    public void setNodeStyle(String nodeStyle) {
        this.nodeStyle = Helper.convertEmptyString(nodeStyle);
    }

    /**
     * <p>
     * Gets the current (only) node CSS style that will be used to draw the node's title.
     * </p>
     *
     * @return A possibly <code>null</code>, never empty (trim'd) string representing the current (only) node CSS style
     *         that will be used to draw the node's title.
     *
     * @see BreadCrumbTrailTag#currentNodeStyle
     */
    public String getCurrentNodeStyle() {
        return currentNodeStyle;
    }

    /**
     * <p>
     * Sets the current (only) node CSS style that will be used to draw the node's title.
     * </p>
     *
     * @param currentNodeStyle A possibly <code>null</code>, possibly empty (trim'd) string representing the current
     *        (only) node CSS style that will be used to draw the node's title. If the value is an empty string, it
     *        will be regarded as <code>null</code>.
     *
     * @see BreadCrumbTrailTag#currentNodeStyle
     */
    public void setCurrentNodeStyle(String currentNodeStyle) {
        this.currentNodeStyle = Helper.convertEmptyString(currentNodeStyle);
    }

    /**
     * <p>
     * Gets the root (only) node CSS style that will be used to draw the node's title.
     * </p>
     *
     * @return A possibly <code>null</code>, never empty (trim'd) string representing the root (only) node CSS style
     *         that will be used to draw the node's title.
     *
     * @see BreadCrumbTrailTag#rootNodeStyle
     */
    public String getRootNodeStyle() {
        return rootNodeStyle;
    }

    /**
     * <p>
     * Sets the root (only) node CSS style that will be used to draw the node's title.
     * </p>
     *
     * @param rootNodeStyle A possibly<code>null</code>, possibly empty (trim'd) string representing the root (only)
     *        node CSS style that will be used to draw the node's title. If the value is an empty string, it will be
     *        regarded as <code>null</code>.
     *
     * @see BreadCrumbTrailTag#rootNodeStyle
     */
    public void setRootNodeStyle(String rootNodeStyle) {
        this.rootNodeStyle = Helper.convertEmptyString(rootNodeStyle);
    }

    /**
     * <p>
     * Gets the mouse over node CSS style that will be used to draw the node's title.
     * </p>
     *
     * @return A possibly <code>null</code>, never empty (trim'd) string representing the mouse over node CSS style
     *         that will be used to draw the node's title.
     *
     * @see BreadCrumbTrailTag#mouseOverStyle
     */
    public String getMouseOverStyle() {
        return mouseOverStyle;
    }

    /**
     * <p>
     * Sets the mouse over node CSS style that will be used to draw the node's title.
     * </p>
     *
     * @param mouseOverStyle A possibly<code>null</code>, possibly empty (trim'd) string representing the mouse over
     *        node CSS style that will be used to draw the node's title. If the value is an empty string, it will be
     *        regarded as <code>null</code>.
     *
     * @see BreadCrumbTrailTag#mouseOverStyle
     */
    public void setMouseOverStyle(String mouseOverStyle) {
        this.mouseOverStyle = Helper.convertEmptyString(mouseOverStyle);
    }

    /**
     * <p>
     * Gets the path override for this specific tag.
     * </p>
     *
     * @return A possibly <code>null</code>, never empty (trim'd) string representing the path override for this
     *         specific tag.
     *
     * @see BreadCrumbTrailTag#pathOverride
     * @since 2.0
     */
    public String getPathOverride() {
        return pathOverride;
    }

    /**
     * <p>
     * Sets the path override for this specific tag.
     * </p>
     *
     * @param pathOverride A possibly<code>null</code>, possibly empty (trim'd) string representing the path override
     *        for this specific tag. If the value is an empty string, it will be regarded as <code>null</code>.
     *
     * @see BreadCrumbTrailTag#pathOverride
     * @since 2.0
     */
    public void setPathOverride(String pathOverride) {
        this.pathOverride = Helper.convertEmptyString(pathOverride);
    }

    /**
     * <p>
     * Gets whether any exceptions should be silently ignore or not.
     * </p>
     *
     * @return <code>true</code> to silently ignore, <code>false</code> to let exception pass through to Jsp engine.
     *
     * @see BreadCrumbTrailTag#silentErrors
     */
    public boolean getSilentErrors() {
        return silentErrors;
    }

    /**
     * <p>
     * Sets whether any exceptions should be silently ignore or not.
     * </p>
     *
     * @param silentErrors <code>true</code> to silently ignore, <code>false</code> to let exception pass through to
     *        Jsp engine.
     *
     * @see BreadCrumbTrailTag#silentErrors
     */
    public void setSilentErrors(boolean silentErrors) {
        this.silentErrors = silentErrors;
    }

    /**
     * <p>
     * Gets the title that will be used if the node is being discovered by this tag.
     * </p>
     *
     * @return A possibly <code>null</code>, never empty (trim'd) string representing the the title that will be used
     *         if the node is being discovered by this tag.
     *
     * @see BreadCrumbTrailTag#title
     * @since 2.0
     */
    public String getTitle() {
        return title;
    }

    /**
     * <p>
     * Sets the title that will be used if the node is being discovered by this tag.
     * </p>
     *
     * @param title A possibly<code>null</code>, possibly empty (trim'd) string representing the title that will be
     *        used if the node is being discovered by this tag. If the value is an empty string, it will be regarded
     *        as <code>null</code>.
     *
     * @see BreadCrumbTrailTag#title
     */
    public void setTitle(String title) {
        this.title = Helper.convertEmptyString(title);
    }

    /**
     * <p>
     * Creates the bread crumb trail. First discover the current node, then discover the current path according to the
     * pre-found current node, and at last format the pre-found current path.
     * </p>
     *
     * <p>
     * <b>Changes in Version 2.0 :</b> this method is fully changed.
     * </p>
     *
     * @return An integer representing what processing to do after this method executes.
     *
     * @throws BreadCrumbException if creating of bread crumb trail failed and silentErrors is <code>false</code>.
     *
     * @since 1.0
     */
    public int doStartTag() throws BreadCrumbException {
        try {
            // discover the current node
            BreadCrumbNode currentNode = this.discoverBreadCrumbNode();

            // discover the current path
            BreadCrumbNode[] path = this.discoverCurrentPath(currentNode);

            if ((path == null) || (path.length == 0)) {
                return SKIP_BODY;
            }

            // format the path
            this.formatPath(path);

            return EVAL_BODY_BUFFERED;
        } catch (BreadCrumbException e) {
            this.throwBreadCrumbException(e);

            return SKIP_BODY;
        }
    }

    /**
     * <p>
     * Discovers the current node.
     * </p>
     *
     * @return the current node.
     *
     * @throws BreadCrumbException if the current node can not be found or it fails to get the current node.
     *
     * @since 2.0
     */
    private BreadCrumbNode discoverBreadCrumbNode() throws BreadCrumbException {
        Object object = this.pageContext.getAttribute(BREADCRUMB_CURRENT_NODE, PageContext.PAGE_SCOPE);

        if (object instanceof BreadCrumbNode) {
            return (BreadCrumbNode) object;
        }

        // use reflection to get the current node
        NodeDiscovery nodeDiscovery = (NodeDiscovery) Helper.createObject(
                OBJECT_FACTORY_NAMESPACE, NodeDiscovery.class);

        if (nodeDiscovery instanceof AbstractNodeDiscovery
                && (((AbstractNodeDiscovery) nodeDiscovery).getTitle() != null)) {
            // set the title if found
            this.setTitle(((AbstractNodeDiscovery) nodeDiscovery).getTitle());
        }

        BreadCrumbNode node = nodeDiscovery.getCurrentNode(this.pageContext);

        if (node == null) {
            throw new BreadCrumbException("The current node can not be found.");
        }

        return node;
    }

    /**
     * <p>
     * Discovers the current path.
     * </p>
     *
     * @param currentNode the current node to discover the current path.
     *
     * @return the current path.
     *
     * @throws BreadCrumbException if it fails to get the current path.
     *
     * @since 2.0
     */
    private BreadCrumbNode[] discoverCurrentPath(BreadCrumbNode currentNode) throws BreadCrumbException {
        PathDiscovery pathDiscovery = null;

        if (this.pathOverride != null) {
            pathDiscovery = new XmlPathDiscovery(this.pathOverride, false);
        } else {
            // If path override is a null, use reflection to get the path
            pathDiscovery = (PathDiscovery) Helper.createObject(OBJECT_FACTORY_NAMESPACE, PathDiscovery.class);
        }

        return pathDiscovery.getPath(currentNode, this.pageContext);
    }

    /**
     * <p>
     * Formats the current path.
     * </p>
     *
     * @param path the current path to format.
     *
     * @throws BreadCrumbException if it fails to format the current path.
     *
     * @since 2.0
     */
    private void formatPath(BreadCrumbNode[] path) throws BreadCrumbException {
        List pathArray = Arrays.asList(path);
        this.iterator = pathArray.listIterator((this.pathDirection == ROOT_TO_CURRENT) ? 0 : pathArray.size());

        // if any of the style variables are non-null, write them to the jsp writer.
        if (shouldPrintSpan()) {
            JspWriter writer = null;

            try {
                writer = pageContext.getOut();
                writer.print("<span>\n");
                this.printStyleSettings(writer);
            } catch (IOException e) {
                throw new BreadCrumbException("IO errors happen in the writing process.", e);
            } finally {
                if (writer != null) {
                    try {
                        writer.close();
                    } catch (IOException e) {
                        // ignore
                    }
                }
            }
        }
    }

    /**
     * <p>
     * Get's the next item from the iterator and set's the attribute information for what needs to be formatted.
     * </p>
     *
     * @since 2.0
     */
    public void doInitBody() {
        boolean isCurrentNode;
        boolean isRootNode;
        BreadCrumbNode node;

        if (this.pathDirection == ROOT_TO_CURRENT) {
            isRootNode = !this.iterator.hasPrevious();
            node = (BreadCrumbNode) this.iterator.next();
            isCurrentNode = !this.iterator.hasNext();
        } else {
            isCurrentNode = !this.iterator.hasNext();
            node = (BreadCrumbNode) this.iterator.previous();
            isRootNode = !this.iterator.hasPrevious();
        }

        String style = null;

        // the priority for style is Root over Current over Node
        if (isRootNode) {
            style = this.rootNodeStyle;
        } else if (isCurrentNode) {
            style = this.currentNodeStyle;
        } else {
            style = this.nodeStyle;
        }

        this.pageContext.setAttribute(BREADCRUMB_FORMAT_NODE, node, PageContext.PAGE_SCOPE);
        this.pageContext.setAttribute(BREADCRUMB_FORMAT_STYLE_CLASS, style, PageContext.PAGE_SCOPE);
    }

    /**
     * <p>
     * Get's the body content text and write's it out and starts again.
     * </p>
     *
     * @return Either SKIP_BODY to stop processing the body or EVAL_BODY_AGAIN to process the next item.
     *
     * @throws BreadCrumbException if IO errors happen in the writing process and silentErrors is <code>false</code>.
     *
     * @since 2.0
     */
    public int doAfterBody() throws BreadCrumbException {
        try {
            JspWriter writer = null;

            try {
                writer = this.getPreviousOut();

                // write the body content
                BodyContent bodyContent = this.getBodyContent();

                if ((bodyContent != null) && (bodyContent.getString().trim().length() != 0)) {
                    writer.print(bodyContent.getString());
                    this.getBodyContent().clear();
                }

                // checks whether the iterator has more element
                boolean hasNext = (this.pathDirection == ROOT_TO_CURRENT) ? this.iterator.hasNext()
                                                                          : this.iterator.hasPrevious();

                if (hasNext) {
                    if (this.pathDirection == ROOT_TO_CURRENT) {
                        iterator.next();
                    } else {
                        iterator.previous();
                    }

                    this.printSeparator(writer);

                    return EVAL_BODY_AGAIN;
                }

                return SKIP_BODY;
            } catch (IOException e) {
                throw new BreadCrumbException("IO errors happen in the writing process.", e);
            } finally {
                if (writer != null) {
                    try {
                        writer.close();
                    } catch (IOException e) {
                        // ignore
                    }
                }
            }
        } catch (BreadCrumbException e) {
            this.throwBreadCrumbException(e);

            return SKIP_BODY;
        }
    }

    /**
     * <p>
     * Prints the style settings out to the specified writer.
     * </p>
     *
     * @return Always EVAL_PAGE.
     *
     * @throws BreadCrumbException if IO errors happen in the writing process and silentErrors is <code>false</code>.
     *
     * @since 2.0
     */
    public int doEndTag() throws BreadCrumbException {
        try {
            if (this.shouldPrintSpan()) {
                JspWriter writer = null;

                try {
                    writer = pageContext.getOut();
                    writer.print("</span>\n");
                } catch (IOException e) {
                    throw new BreadCrumbException("IO errors happen in the writing process.", e);
                } finally {
                    if (writer != null) {
                        try {
                            writer.close();
                        } catch (IOException e) {
                            // ignore
                        }
                    }
                }
            }
        } catch (BreadCrumbException e) {
            this.throwBreadCrumbException(e);
        }

        return EVAL_PAGE;
    }

    /**
     * <p>
     * Prints the style settings out to the specified writer.
     * </p>
     *
     * <p>
     * <b>Changes in Version 2.0 :</b> Changed method to use the argument rather than class variable and added
     * <code>IllegalArgumentException</code> if argument is <code>null</code>.
     * </p>
     *
     * @param out The non-null JspWriter to write the style settings to.
     *
     * @throws IllegalArgumentException if the out is <code>null</code>.
     * @throws IOException if an io occurs writing out the style.
     *
     * @since 1.0
     */
    private void printStyleSettings(JspWriter out) throws IOException {
        Helper.validateNotNull(out, "out");
        out.print("<style>\n");

        if (this.pathSeparatorStyle != null) {
            printStyle(out, "path-separator-style", this.pathSeparatorStyle);
        }

        if (this.nodeStyle != null) {
            printStyle(out, ".node-style a:link", this.nodeStyle);
        }

        if (this.currentNodeStyle != null) {
            printStyle(out, ".current-node-style a:link", this.currentNodeStyle);
        }

        if (this.rootNodeStyle != null) {
            printStyle(out, ".root-node-style a:link", this.rootNodeStyle);
        }

        if (this.mouseOverStyle != null) {
            printStyle(out, ".node-style a:hover", this.mouseOverStyle);

            if (this.currentNodeStyle != null) {
                printStyle(out, ".current-node-style a:hover", this.mouseOverStyle);
            }

            if (this.rootNodeStyle != null) {
                printStyle(out, ".root-node-style a:hover", this.mouseOverStyle);
            }
        }

        out.print("</style>\n");
    }

    /**
     * <p>
     * Checks whether the style settings should be printed. If any of the style settings is defined, the return value
     * will be <code>true</code>.
     * </p>
     *
     * @return whether the style settings should be printed.
     */
    private boolean shouldPrintSpan() {
        if (this.pathSeparatorStyle != null) {
            return true;
        }

        if (this.nodeStyle != null) {
            return true;
        }

        if (this.currentNodeStyle != null) {
            return true;
        }

        if (this.rootNodeStyle != null) {
            return true;
        }

        if (this.mouseOverStyle != null) {
            return true;
        }

        return false;
    }

    /**
     * <p>
     * Prints the specified style out to the specified writer.
     * </p>
     *
     * <p>
     * <b>Changes in Version 2.0 :</b> Changed method to use the argument rather than class variable and added
     * <code>IllegalArgumentException</code> if argument is <code>null</code>.
     * </p>
     *
     * @param out The non-null JspWriter to write the style settings to.
     * @param style The non-null, non-empty (trim'd) string representing the style to write.
     * @param value The non-null, non-empty (trim'd) string representing the value of the style to write.
     *
     * @throws IllegalArgumentException if any argument is <code>null</code> or any string argument is an empty
     *         (trim'd) string.
     * @throws IOException if an io occurs writing out the style.
     *
     * @since 1.0
     */
    private void printStyle(JspWriter out, String style, String value) throws IOException {
        Helper.validateNotNull(out, "out");
        Helper.validateString(style, "style");
        Helper.validateString(value, "value");
        out.print(style);
        out.print(" {");
        out.print(value);
        out.print("}\n");
    }

    /**
     * <p>
     * Prints the separator (with the separator style) to the specified writer.
     * </p>
     *
     * <p>
     * <b>Changes in Version 2.0 :</b> Changed method to use the argument rather than class variable and added
     * <code>IllegalArgumentException</code> if argument is <code>null</code>.
     * </p>
     *
     * @param out The non-null JspWriter to write the style settings to.
     *
     * @throws IllegalArgumentException if the out is <code>null</code>.
     * @throws IOException if an io occurs writing out the style.
     *
     * @since 1.0
     */
    private void printSeparator(JspWriter out) throws IOException {
        out.print("<span class=path-separator-style>");
        out.print(this.pathSeparator);
        out.print("</span>\n");
    }

    /**
     * <p>
     * Throws a <code>BreadCrumbException</code> with given message if <code>silentErrors</code> == <code>false</code>.
     * </p>
     *
     * @param breadCrumbException the real <code>BreadCrumbException</code>.
     *
     * @throws BreadCrumbException if <code>silentErrors</code> == <code>false</code>.
     *
     * @since 2.0
     */
    private void throwBreadCrumbException(BreadCrumbException breadCrumbException) throws BreadCrumbException {
        if (!silentErrors) {
            throw breadCrumbException;
        }
    }

    /**
     * Get the string in <code>ConfigManager</code> with specified namespace and name.
     *
     * @param name the name of the config string.
     * @param defaultValue the default value if the config value is missing.
     *
     * @return the config string in <code>ConfigManager</code>.
     *
     * @since 2.0
     */
    private String getStringPropertyValue(String name, String defaultValue) {
        try {
            String configValue = ConfigManager.getInstance().getString(DEFAULT_NAMESPACE, name);

            if (configValue == null) {
                return defaultValue;
            }

            return configValue;
        } catch (UnknownNamespaceException une) {
            return defaultValue;
        }
    }

    /**
     * Get the int value in <code>ConfigManager</code> with specified namespace and name.
     *
     * @param name the name of the config int value.
     * @param defaultValue the default value if the config value is missing.
     *
     * @return the config string in <code>ConfigManager</code>.
     *
     * @since 2.0
     */
    private int getIntPropertyValue(String name, int defaultValue) {
        String stringValue = getStringPropertyValue(name, "null");

        if (stringValue == null) {
            return defaultValue;
        }

        try {
            return Integer.parseInt(stringValue);
        } catch (NumberFormatException e) {
            return defaultValue;
        }
    }

    /**
     * Get the boolean value in <code>ConfigManager</code> with specified namespace and name.
     *
     * @param name the name of the config boolean value.
     * @param defaultValue the default value if the config value is missing.
     *
     * @return the config string in <code>ConfigManager</code>.
     *
     * @since 2.0
     */
    private boolean getBooleanPropertyValue(String name, boolean defaultValue) {
        String stringValue = getStringPropertyValue(name, "null");

        if (stringValue == null) {
            return defaultValue;
        }

        return Boolean.getBoolean(stringValue);
    }
}
