/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import com.topcoder.util.objectfactory.InvalidClassSpecificationException;
import com.topcoder.util.objectfactory.ObjectFactory;
import com.topcoder.util.objectfactory.impl.ConfigManagerSpecificationFactory;
import com.topcoder.util.objectfactory.impl.IllegalReferenceException;
import com.topcoder.util.objectfactory.impl.SpecificationConfigurationException;

import com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery.NodeMatcher;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * Defines utilities for persistence classes.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0
 */
public final class Helper {
    /**
     * Private constructor to prevent this class be instantiated.
     */
    private Helper() {
        // empty
    }

    /**
     * <p>
     * Validates the value of a variable with type <code>Object</code>. The value cannot be <code>null</code>.
     * </p>
     *
     * @param value value of the variable.
     * @param name name of the variable.
     *
     * @throws IllegalArgumentException if <code>value</code> is <code>null</code>.
     */
    public static void validateNotNull(Object value, String name) {
        if (value == null) {
            throw new IllegalArgumentException(name + " cannot be null.");
        }
    }

    /**
     * <p>
     * Validates the value of a string variable. The value cannot be empty string, but may be <code>null</code>.
     * </p>
     *
     * @param value value of the variable.
     * @param name name of the variable.
     *
     * @throws IllegalArgumentException if <code>value</code> is empty string.
     */
    public static void validateNotEmpty(String value, String name) {
        if (value != null) {
            if (value.trim().length() == 0) {
                throw new IllegalArgumentException(name + " cannot be empty string.");
            }
        }
    }

    /**
     * <p>
     * Validates the value of a string variable. The value cannot be <code>null</code> or empty string.
     * </p>
     *
     * @param value value of the variable.
     * @param name name of the variable.
     *
     * @throws NullPointerException
     * @throws IllegalArgumentException if <code>value</code> is <code>null</code> or empty string.
     */
    public static void validateString(String value, String name) {
        validateNotNull(value, name);
        validateNotEmpty(value, name);
    }

    /**
     * Converts the empty string to <code>null</code>. If the valud is not an empty string, the original string will be
     * returned.
     *
     * @param value the string value to convert.
     *
     * @return the converted string value.
     */
    public static String convertEmptyString(String value) {
        if ((value != null) && (value.trim().length() == 0)) {
            return null;
        }

        return value;
    }

    /**
     * Builds the reverse mapping of the given map.
     *
     * @param siteMap the given map to build a reverse mapping.
     *
     * @return the reverse mapping of the given map.
     */
    public static Map getCurrentReverseSiteMap(Map siteMap) {
        Map currentReverseSiteMap = new HashMap();

        for (Iterator iter = siteMap.entrySet().iterator(); iter.hasNext();) {
            Map.Entry entry = (Map.Entry) iter.next();
            NodeMatcher nodeMatcher = (NodeMatcher) entry.getKey();
            List nodeMatchers = (List) entry.getValue();

            for (Iterator nodeMatchersIterator = nodeMatchers.iterator(); nodeMatchersIterator.hasNext();) {
                NodeMatcher nodeMatcherChild = (NodeMatcher) nodeMatchersIterator.next();
                List nodeMatcherParents = (List) currentReverseSiteMap.get(nodeMatcherChild);

                if (nodeMatcherParents == null) {
                    nodeMatcherParents = new ArrayList();
                    currentReverseSiteMap.put(nodeMatcherChild, nodeMatcherParents);
                }

                nodeMatcherParents.add(nodeMatcher);
            }
        }

        // sort the nodeMatcherParents list in order to determine the first when more than one shortest path exist
        for (Iterator iter = currentReverseSiteMap.values().iterator(); iter.hasNext();) {
            List nodeMatcherParents = (List) iter.next();
            Collections.sort(nodeMatcherParents, new NodeMatcherComparator());
        }

        return currentReverseSiteMap;
    }

    /**
     * Get the object of the specified class from the object factory.
     *
     * @param namespace the namespace for the object factory.
     * @param expectedClass the expected class.
     *
     * @return the object of the specified class from the object factory.
     *
     * @throws BreadCrumbException if it fails to create the expected class from object factory.
     */
    public static Object createObject(String namespace, Class expectedClass) throws BreadCrumbException {
        try {
            ObjectFactory factory = new ObjectFactory(new ConfigManagerSpecificationFactory(namespace));

            return factory.createObject(expectedClass);
        } catch (InvalidClassSpecificationException e) {
            throw new BreadCrumbException("Can not create the expected class from object factory.", e);
        } catch (SpecificationConfigurationException e) {
            throw new BreadCrumbException("The error occurs when getting from ConfigManager.", e);
        } catch (IllegalReferenceException e) {
            throw new BreadCrumbException("If the config is invalid somewhere, for example contains loop.", e);
        }
    }

    /**
     * A comparator for sorting the <code>NodeMatcher</code> collection. This sorting can help to determine the first
     * when more than one shortest path exist.
     *
     * @author TCSDEVELOPER
     * @version 2.0
     */
    private static class NodeMatcherComparator implements Comparator {
        /**
         * Compares the two objects.
         *
         * @param object1 the first object.
         * @param object2 the second object.
         *
         * @return the compare result.
         */
        public int compare(Object object1, Object object2) {
            int ret = ((NodeMatcher) object1).getTitle().compareTo(((NodeMatcher) object2).getTitle());

            if (ret != 0) {
                return ret;
            }

            return ((NodeMatcher) object1).getUrl().compareTo(((NodeMatcher) object2).getUrl());
        }
    }
}
