/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.nodediscovery;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbTrailTag;
import com.topcoder.web.ui.tag.breadcrumbtrail.Helper;

import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.BodyContent;
import javax.servlet.jsp.tagext.BodyTagSupport;


/**
 * <p>
 * This class provides a tag (only) implementation of node discovery that will allow the user to hardcode the url into
 * the body of this tag and specify the title in the attributes. That hardcoded information is then read by this tag,
 * combined with the title to make a <code>BreadCrumbNode</code> and that node is put in a page level attribute called
 * <code>BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE</code>.
 * </p>
 *
 * <p>
 * <strong>Thread Safety:</strong> This class is thread safe in the context of the JSP rendering process
 * </p>
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 */
public class InlineTextNodeDiscoveryTag extends BodyTagSupport {
    /**
     * <p>
     * Represents the title associated with the node.
     * </p>
     *
     * <p>
     * This variable is set by the constructor initially to <code>null</code> and is mutable. This value can be
     * <code>null</code> if the title should default to the title found in the <code>PathDiscovery</code>
     * implementation. This value should not be <code>null</code> if it's a node that is part of a path returned by
     * the <code>PathDiscovery</code> implementation. This value should never be an empty string.
     * </p>
     */
    private String title = null;

    /**
     * <p>
     * Creates a new instance of <code>InlineTextNodeDiscoveryTag</code> class.
     * </p>
     */
    public InlineTextNodeDiscoveryTag() {
        super();
    }

    /**
     * <p>
     * Starts the processing of this tag by telling the support class to evaluate the body.
     * </p>
     *
     * @return Always EVAL_BODY_BUFFERED.
     */
    public int doStartTag() {
        return EVAL_BODY_BUFFERED;
    }

    /**
     * <p>
     * Processes the body content. This tag will simply copy the body content, create a <code>BreadCrumbNode</code> and
     * set it to the attributes as a page level attribute called
     * <code>BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE</code> then clear the body. Please note that the body is never
     * written out as this tag doesn't render anything.
     * </p>
     *
     * @return Always SKIP_BODY.
     *
     * @throws BreadCrumbException if the body contains nothing.
     */
    public int doAfterBody() throws BreadCrumbException {
        BodyContent content = this.getBodyContent();

        if (content == null) {
            throw new BreadCrumbException("The body contains nothing.");
        }

        if (content.getString().trim().length() == 0) {
            throw new BreadCrumbException("The body contains empty content.");
        }

        BreadCrumbNode breadCrumbNode = new BreadCrumbNode(content.getString(), getTitle());
        this.pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE,
                breadCrumbNode, PageContext.PAGE_SCOPE);

        return SKIP_BODY;
    }

    /**
     * <p>
     * Gets the title of page represented by the node.
     * </p>
     *
     * @return A possibly <code>null</code>, non-empty (trim'd) string representing the title of page represented by
     *         the node.
     */
    public String getTitle() {
        return title;
    }

    /**
     * <p>
     * Sets the title of page represented by the node.
     * </p>
     *
     * @param title the title of page represented by the node. If the title is an empty string, it will be regarded as
     *        <code>null</code>.
     */
    public void setTitle(String title) {
        this.title = Helper.convertEmptyString(title);
    }
}
