/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.Helper;
import com.topcoder.web.ui.tag.breadcrumbtrail.PathDiscovery;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.servlet.jsp.PageContext;


/**
 * <p>
 * This implementation of the <code>PathDiscovery</code> will attempt to create a static site map and then apply
 * dynamic information to that site map. The dynamic information can either be in matching nodes with different query
 * strings or in new nodes that will be assumed to map to the last known node.
 * </p>
 *
 * <p>
 * <strong>Thread Safety:</strong> Implementations of this interface should be thread safe.
 * </p>
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 */
public class DynamicPathDiscovery implements PathDiscovery {
    /**
     * <p>
     * Represents the session scope attribute name that will contain the current known path. This constant will be used
     * by this class to retrieve the current path. The current path will be of NodePair[] type representing the know
     * path from the root (0th index) to the prior page (lenght-1th index) .
     * </p>
     */
    public static final String CURRENT_PATH = DynamicPathDiscovery.class.getName() + ".currentPath";

    /**
     * <p>
     * Represents the session scope attribute name that will contain the site map. This constant will be used by this
     * class to retrieve the site map. The site map will be a <code>Map</code> with <code>NodeMatcher</code> as the
     * key and a list of <code>NodeMatcher</code> as the value and can be empty. This map represents the
     * <code>NodeMatcher</code> parent to it's children NodeMatcher(s) if any. The list can be empty if there are no
     * children of the parent (ie a leaf node).
     * </p>
     */
    public static final String CURRENT_SITEMAP = DynamicPathDiscovery.class.getName() + ".siteMap";

    /**
     * <p>
     * Represents the session scope attribute name that will contain the reverse site map. This constant will be used
     * by this class to retrieve the reverse site map. The reverse site map will be a <code>Map</code> with
     * <code>NodeMatcher</code> as the key and a list of <code>NodeMatcher</code> as the value and can not be empty.
     * This map represents the <code>NodeMatcher</code> child to it's parents NodeMatcher(s) if any.
     * </p>
     */
    public static final String CURRENT_REVERSE_SITEMAP = DynamicPathDiscovery.class.getName()
        + ".currentReverseSiteMap";

    /**
     * <p>
     * Creates a new instance of <code>DynamicPathDiscovery</code> class.
     * </p>
     */
    public DynamicPathDiscovery() {
        // do nothing here
    }

    /**
     * <p>
     * This method attempts to determine the page from the current node to the root and return that path.
     * </p>
     *
     * @param currentNode A non-null current node.
     * @param pageContext A non-null page context.
     *
     * @return A null if the path can't be found or a non-empty array representing the path from root node (0th index)
     *         to current node (size-1th index).
     *
     * @throws IllegalArgumentException if either arg is null.
     * @throws BreadCrumbException if an exception occurred while processing.
     */
    public BreadCrumbNode[] getPath(BreadCrumbNode currentNode, PageContext pageContext) throws BreadCrumbException {
        // get the current path
        Object object = pageContext.getAttribute(CURRENT_PATH, PageContext.SESSION_SCOPE);
        NodePair[] currentPath = (object instanceof NodePair[]) ? (NodePair[]) object : null;

        // get the site map
        object = pageContext.getAttribute(CURRENT_SITEMAP, PageContext.SESSION_SCOPE);

        Map siteMap = (object instanceof Map) ? (Map) object : null;

        if (siteMap == null) {
            // clear the current path
            currentPath = null;

            // use reflection to get the site map
            siteMap = ((SiteDiscovery) Helper.createObject(DynamicPathDiscovery.class.getName(),
                    SiteDiscovery.class)).getSiteMap();
            pageContext.setAttribute(CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);
        }

        // is current node on the current path
        if (currentPath != null) {
            for (int i = currentPath.length - 1; i >= 0; i--) {
                if (((currentPath[i].getNodeMatcher() == null)
                        && !currentPath[i].getNode().getUrl().equals(currentNode.getUrl()))
                        || !currentPath[i].getNodeMatcher().matches(currentNode)) {
                    continue;
                }

                NodePair[] newPath = new NodePair[i + 1];
                System.arraycopy(currentPath, 0, newPath, 0, i + 1);

                return saveState(pageContext, newPath, siteMap, currentNode);
            }
        }

        // if the node is one off the last node
        if ((currentPath != null) && (siteMap != null)) {
            NodePair lastPair = null;

            for (int i = currentPath.length - 1; i >= 0; i--) {
                // get the last node that has a non-null nodeMatcher
                if (currentPath[i].getNodeMatcher() != null) {
                    lastPair = currentPath[i];

                    break;
                }
            }

            if (lastPair != null) {
                // get the node matcher list from the site map
                List nodeMatchers = (List) siteMap.get(lastPair.getNodeMatcher());

                if (nodeMatchers != null) {
                    for (Iterator iter = nodeMatchers.iterator(); iter.hasNext();) {
                        NodeMatcher nodeMatcher = (NodeMatcher) iter.next();

                        if (nodeMatcher.matches(currentNode)) {
                            NodePair[] newPath = new NodePair[currentPath.length + 1];
                            System.arraycopy(currentPath, 0, newPath, 0, currentPath.length);
                            newPath[currentPath.length] = new NodePair(currentNode, nodeMatcher);

                            return saveState(pageContext, newPath, siteMap, currentNode);
                        }
                    }
                }
            }
        }

        // get the shortest path to a root node
        NodeMatcher[] shortestPath = getShortestPath(pageContext, siteMap, currentNode, currentPath);

        if (shortestPath != null) {
            // if the shortest path is found, its length will alway be greater than 0
            int startIdx = 0;
            NodePair[] newPath = new NodePair[shortestPath.length];

            if (currentPath != null) {
                for (; (startIdx < shortestPath.length) && (startIdx < currentPath.length); startIdx++) {
                    if (!shortestPath[startIdx].matches(currentPath[startIdx].getNode())) {
                        break;
                    }
                }

                System.arraycopy(currentPath, 0, newPath, 0, startIdx);
            }

            for (int i = startIdx; i < shortestPath.length; i++) {
                newPath[i] = new NodePair(new BreadCrumbNode(shortestPath[i].getUrl(), shortestPath[i].getTitle()),
                        shortestPath[i]);
            }

            return saveState(pageContext, newPath, siteMap, currentNode);
        }

        if (currentPath == null) {
            // we have no idea what the path is.
            return null;
        }

        // a 'discovered' node and append current node to current path
        NodePair[] newPath = new NodePair[currentPath.length + 1];
        System.arraycopy(currentPath, 0, newPath, 0, currentPath.length);
        newPath[currentPath.length] = new NodePair(currentNode, null);

        return saveState(pageContext, newPath, siteMap, currentNode);
    }

    /**
     * <p>
     * Find the shortest path from the site map. The shortest path will be a nodeMatcher array from root nodeMatcher
     * [index 0] to whatever node matcher matches the current node. The totle time complexity is O(n) where n is the
     * number of all the nodes in the site map.
     * </p>
     *
     * @param pageContext the pageContext that can read some session attributes.
     * @param siteMap the site map.
     * @param currentNode the current node.
     * @param currentPath the current path.
     *
     * @return the shortest path.
     */
    private NodeMatcher[] getShortestPath(PageContext pageContext, Map siteMap, BreadCrumbNode currentNode,
        NodePair[] currentPath) {
        if (siteMap == null) {
            return null;
        }

        // get the reverse mapping of the site map which will help to find all the nodeMatchers' parents
        Map currentReverseSiteMap = null;
        Object object = pageContext.getAttribute(CURRENT_REVERSE_SITEMAP, PageContext.SESSION_SCOPE);

        if (!(object instanceof Map)) {
            currentReverseSiteMap = Helper.getCurrentReverseSiteMap(siteMap);
            pageContext.setAttribute(CURRENT_REVERSE_SITEMAP, currentReverseSiteMap, PageContext.SESSION_SCOPE);
        } else {
            currentReverseSiteMap = (Map) object;
        }

        // try to find the shortest path
        List shortestPath = null;
        int[] mostCommonality = new int[1];
        int[] tempCommonality = new int[1];

        for (Iterator iter = siteMap.keySet().iterator(); iter.hasNext();) {
            NodeMatcher nodeMatcher = (NodeMatcher) iter.next();

            if (nodeMatcher.matches(currentNode)) {
                List tempPath = getShortestPath(nodeMatcher, currentReverseSiteMap, currentPath, tempCommonality);

                if (tempPath != null) {
                    // update the shortest path
                    shortestPath = updateShortestPath(shortestPath, mostCommonality, tempPath, tempCommonality);
                }
            }
        }

        if (shortestPath != null) {
            return (NodeMatcher[]) shortestPath.toArray(new NodeMatcher[shortestPath.size()]);
        }

        // no shortest path can be found
        return null;
    }

    /**
     * <p>
     * Find the shortest path from the site map. The shortest path will be a nodeMatcher array from root nodeMatcher
     * [index 0] to whatever node matcher matches the current node.
     * </p>
     *
     * @param originalMatcher the original matcher.
     * @param currentReverseSiteMap the current reverse site map.
     * @param currentPath the current path.
     * @param mostCommonality the array to store the most commonality value.
     *
     * @return the shortest path.
     */
    private List getShortestPath(NodeMatcher originalMatcher, Map currentReverseSiteMap, NodePair[] currentPath,
        int[] mostCommonality) {
        int[] tempCommonality = new int[1];

        // use bfs to get the shortest path
        Map child = new HashMap();
        // The totle time complexity is O(n) where n is the number of all the nodes in the site map since once a
        // node is reached, it will never be reached again
        List shortestPath = null;

        // use LinkedList to be more efficient
        List queue = new LinkedList();
        queue.add(originalMatcher);
        child.put(originalMatcher, originalMatcher);

        while (!queue.isEmpty()) {
            NodeMatcher matcher = (NodeMatcher) ((LinkedList) queue).removeFirst();
            List parents = (List) currentReverseSiteMap.get(matcher);

            if (parents == null) {
                // if it is the root, return the shortest path
                List tempPath = getPath(originalMatcher, matcher, child);

                if (currentPath == null) {
                    // no current path exists, return the first shortest path that found
                    return tempPath;
                }

                // get the commonality
                tempCommonality[0] = this.getCommonality(currentPath, tempPath);

                // update the shortest path
                shortestPath = updateShortestPath(shortestPath, mostCommonality, tempPath, tempCommonality);
            } else {
                for (Iterator iter = parents.iterator(); iter.hasNext();) {
                    NodeMatcher nodeMatcher = (NodeMatcher) iter.next();

                    if (!child.containsKey(nodeMatcher)) {
                        queue.add(nodeMatcher);
                        child.put(nodeMatcher, matcher);
                    } else if (currentPath != null) {
                        // we update the node in the queue with more mostCommonality
                        int preCommonality = this.getCommonality(currentPath,
                                this.getPath(originalMatcher, nodeMatcher, child));
                        List list = this.getPath(originalMatcher, matcher, child);
                        list.add(nodeMatcher);

                        int nowCommonality = this.getCommonality(currentPath, list);

                        if (nowCommonality > preCommonality) {
                            child.put(nodeMatcher, matcher);
                        }
                    }
                }
            }
        }

        return shortestPath;
    }

    /**
     * Build the path from the child map.
     *
     * @param orginalMatcher the orginal matcher.
     * @param matcher the current matcher.
     * @param child the child map.
     *
     * @return the path built from the map.
     */
    private List getPath(NodeMatcher orginalMatcher, NodeMatcher matcher, Map child) {
        List path = new ArrayList();

        while (!matcher.equals(orginalMatcher)) {
            path.add(matcher);
            matcher = (NodeMatcher) child.get(matcher);
        }

        path.add(orginalMatcher);

        return path;
    }

    /**
     * Gets the commonality value for a given path.
     *
     * @param currentPath the current path.
     * @param path the given path to get the commonality.
     *
     * @return the commonality value for a given path.
     */
    private int getCommonality(NodePair[] currentPath, List path) {
        int count = 0;

        for (Iterator iter = path.iterator(); iter.hasNext();) {
            NodeMatcher matcher = (NodeMatcher) iter.next();

            for (int i = 0; i < currentPath.length; i++) {
                if (matcher.matches(currentPath[i].getNode())) {
                    count++;

                    break;
                }
            }
        }

        return count;
    }

    /**
     * Updates the shortest path if the new path is shorter or more commonality.
     *
     * @param shortestPath the current shortest path.
     * @param mostCommonality the current most commonality.
     * @param tempPath the current path
     * @param tempCommonality the current commonality.
     *
     * @return the new shortest path.
     */
    private List updateShortestPath(List shortestPath, int[] mostCommonality, List tempPath, int[] tempCommonality) {
        if ((shortestPath == null) || (tempPath.size() < shortestPath.size())) {
            mostCommonality[0] = tempCommonality[0];

            return tempPath;
        }

        if ((tempPath.size() == shortestPath.size()) && (tempCommonality[0] > mostCommonality[0])) {
            mostCommonality[0] = tempCommonality[0];

            return tempPath;
        }

        return shortestPath;
    }

    /**
     * Saves the state of the current path and return the current shortest path.
     *
     * @param pageContext the page context.
     * @param currentPath the current path.
     * @param siteMap the site map.
     * @param currentNode the current node.
     *
     * @return the current shortest path.
     */
    private BreadCrumbNode[] saveState(PageContext pageContext, NodePair[] currentPath, Map siteMap,
        BreadCrumbNode currentNode) {
        pageContext.setAttribute(CURRENT_PATH, currentPath, PageContext.SESSION_SCOPE);
        pageContext.setAttribute(CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        if ((currentPath == null) || (currentPath.length == 0)) {
            return null;
        }

        BreadCrumbNode[] breadCrumbNodes = new BreadCrumbNode[currentPath.length];

        // The last element in the path should be set to currentNode instead.
        for (int i = 0; i < (currentPath.length - 1); i++) {
            breadCrumbNodes[i] = currentPath[i].getNode();
        }

        if (currentNode.getTitle() != null) {
            breadCrumbNodes[currentPath.length - 1] = currentNode;
        } else if (currentPath[currentPath.length - 1].getNodeMatcher() != null) {
            breadCrumbNodes[currentPath.length - 1] = new BreadCrumbNode(currentNode.getUrl(),
                    currentPath[currentPath.length - 1].getNodeMatcher().getTitle());
        } else {
            breadCrumbNodes[currentPath.length - 1] = new BreadCrumbNode(currentNode.getUrl(),
                    null);
        }

        return breadCrumbNodes;
    }
}
