/*
 * Copyright (c) 2004, TopCoder, Inc. All rights reserved
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.UnitTestHelper;

import org.apache.cactus.JspTestCase;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.jsp.PageContext;


/**
 * <p>
 * Tests functionality and error cases of <code>DynamicPathDiscovery</code> class.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0
 */
public class DynamicPathDiscoveryUnitTest extends JspTestCase {
    /** Represents the <code>DynamicPathDiscovery</code> instance used for testing. */
    private DynamicPathDiscovery pathDiscovery = null;

    /**
     * <p>
     * Sets up the test environment. Adds the default config file. The test instance is created.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    protected void setUp() throws Exception {
        UnitTestHelper.addConfig("ObjectFactoryConfig.xml");
        pathDiscovery = new DynamicPathDiscovery();
    }

    /**
     * Removes the configuration if it exists.
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    protected void tearDown() throws Exception {
        UnitTestHelper.clearConfig();
    }

    /**
     * <p>
     * Tests the accuracy of constructor <code>DynamicPathDiscovery()</code>.
     * </p>
     */
    public void testDynamicPathDiscovery_Accuracy() {
        assertNotNull("The DynamicPathDiscovery instance should be created.", pathDiscovery);
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPath(BreadCrumbNode, PageContext)</code>. The siteMap will be retrieved
     * via reflection.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetPath_NoSiteMapAccuracy() throws Exception {
        BreadCrumbNode currentNode = new BreadCrumbNode("/leafUrl", "leafTitle0");

        // check the result
        BreadCrumbNode[] path = pathDiscovery.getPath(currentNode, pageContext);
        assertNotNull("The shortest path should exist.", path);
        assertEquals("The shortest path should be correct.", 3, path.length);
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPath(BreadCrumbNode, PageContext)</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetPath_HasCurrentPathAccuracy1() throws Exception {
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("C");

        // build the siteMap
        Map siteMap = new HashMap();
        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        // build the current path, the current path is A->B->C
        NodePair[] currentPath = new NodePair[] {
            new NodePair(UnitTestHelper.buildBreadCrumbNode("A"), UnitTestHelper.buildNodeMatcher("A")),
            new NodePair(UnitTestHelper.buildBreadCrumbNode("B"), UnitTestHelper.buildNodeMatcher("B")),
            new NodePair(UnitTestHelper.buildBreadCrumbNode("C"), UnitTestHelper.buildNodeMatcher("C"))};
        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_PATH, currentPath, PageContext.SESSION_SCOPE);

        Object object = pageContext.getAttribute(DynamicPathDiscovery.CURRENT_PATH, PageContext.SESSION_SCOPE);
        assertEquals(true, object instanceof NodePair[]);

        // check the result
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
                UnitTestHelper.buildBreadCrumbNode("A"), UnitTestHelper.buildBreadCrumbNode("B"),
                UnitTestHelper.buildBreadCrumbNode("C")};

        UnitTestHelper.assertNodeEquals("The path returned should be correct.", expected,
            pathDiscovery.getPath(currentNode, pageContext));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPath(BreadCrumbNode, PageContext)</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetPath_HasCurrentPathAccuracy2() throws Exception {
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("B");

        // build the siteMap
        Map siteMap = new HashMap();
        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        // build the current path, the current path is A->B->C
        NodePair[] currentPath = new NodePair[] {
            new NodePair(UnitTestHelper.buildBreadCrumbNode("A"), UnitTestHelper.buildNodeMatcher("A")),
            new NodePair(UnitTestHelper.buildBreadCrumbNode("B"), UnitTestHelper.buildNodeMatcher("B")),
            new NodePair(UnitTestHelper.buildBreadCrumbNode("C"), UnitTestHelper.buildNodeMatcher("C"))};
        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_PATH, currentPath, PageContext.SESSION_SCOPE);

        // check the result
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
                UnitTestHelper.buildBreadCrumbNode("A"), UnitTestHelper.buildBreadCrumbNode("B")};

        UnitTestHelper.assertNodeEquals("The path returned should be correct.", expected,
            pathDiscovery.getPath(currentNode, pageContext));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPath(BreadCrumbNode, PageContext)</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetPath_HasCurrentPathAccuracy3() throws Exception {
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("D");

        // build the siteMap
        Map siteMap = new HashMap();
        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        // build the current path, the current path is A->B->C
        NodePair[] currentPath = new NodePair[] {
            new NodePair(UnitTestHelper.buildBreadCrumbNode("A"), UnitTestHelper.buildNodeMatcher("A")),
            new NodePair(UnitTestHelper.buildBreadCrumbNode("B"), UnitTestHelper.buildNodeMatcher("B")),
            new NodePair(UnitTestHelper.buildBreadCrumbNode("C"), UnitTestHelper.buildNodeMatcher("C"))};
        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_PATH, currentPath, PageContext.SESSION_SCOPE);

        // check the result
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
                UnitTestHelper.buildBreadCrumbNode("A"), UnitTestHelper.buildBreadCrumbNode("B"),
                UnitTestHelper.buildBreadCrumbNode("C"), UnitTestHelper.buildBreadCrumbNode("D")};

        UnitTestHelper.assertNodeEquals("The path returned should be correct.", expected,
            pathDiscovery.getPath(currentNode, pageContext));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPath(BreadCrumbNode, PageContext)</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetPath_HasCurrentPathAccuracy4() throws Exception {
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("D");

        // build the siteMap
        Map siteMap = new HashMap();

        // the map will be C->D
        siteMap.put(UnitTestHelper.buildNodeMatcher("C"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("D")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("D"), new ArrayList());
        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        // build the current path, the current path is A->B->C
        NodePair[] currentPath = new NodePair[] {
            new NodePair(UnitTestHelper.buildBreadCrumbNode("A"), UnitTestHelper.buildNodeMatcher("A")),
            new NodePair(UnitTestHelper.buildBreadCrumbNode("B"), UnitTestHelper.buildNodeMatcher("B")),
            new NodePair(UnitTestHelper.buildBreadCrumbNode("C"), UnitTestHelper.buildNodeMatcher("C"))};
        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_PATH, currentPath, PageContext.SESSION_SCOPE);

        // check the result
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
                UnitTestHelper.buildBreadCrumbNode("A"), UnitTestHelper.buildBreadCrumbNode("B"),
                UnitTestHelper.buildBreadCrumbNode("C"), UnitTestHelper.buildBreadCrumbNode("D")};

        UnitTestHelper.assertNodeEquals("The path returned should be correct.", expected,
            pathDiscovery.getPath(currentNode, pageContext));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPath(BreadCrumbNode, PageContext)</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetPath_HasCurrentPathAccuracy5() throws Exception {
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("D");

        // build the siteMap
        Map siteMap = new HashMap();

        // the map will be B->D
        siteMap.put(UnitTestHelper.buildNodeMatcher("B"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("D")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("D"), new ArrayList());
        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        // build the current path, the current path is A->B->C
        NodePair[] currentPath = new NodePair[] {
            new NodePair(UnitTestHelper.buildBreadCrumbNode("A"), UnitTestHelper.buildNodeMatcher("A")),
            new NodePair(UnitTestHelper.buildBreadCrumbNode("B"), UnitTestHelper.buildNodeMatcher("B")),
            new NodePair(UnitTestHelper.buildBreadCrumbNode("C"), UnitTestHelper.buildNodeMatcher("C"))};
        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_PATH, currentPath, PageContext.SESSION_SCOPE);

        // check the result
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
                UnitTestHelper.buildBreadCrumbNode("B"), UnitTestHelper.buildBreadCrumbNode("D")};

        UnitTestHelper.assertNodeEquals("The path returned should be correct.", expected,
            pathDiscovery.getPath(currentNode, pageContext));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPath(BreadCrumbNode, PageContext)</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetPath_HasCurrentPathAccuracy6() throws Exception {
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("D");

        // build the siteMap
        Map siteMap = new HashMap();

        // the map will be A->B->C->D, A->E->F->D
        siteMap.put(UnitTestHelper.buildNodeMatcher("A"),
            Arrays.asList(
                new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("B"), UnitTestHelper.buildNodeMatcher("E")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("B"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("C"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("D")}));

        siteMap.put(UnitTestHelper.buildNodeMatcher("E"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("F")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("F"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("D")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("D"), new ArrayList());
        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        // build the current path, the current path is A->F->G
        NodePair[] currentPath = new NodePair[] {
            new NodePair(UnitTestHelper.buildBreadCrumbNode("A"), UnitTestHelper.buildNodeMatcher("A")),
            new NodePair(UnitTestHelper.buildBreadCrumbNode("F"), UnitTestHelper.buildNodeMatcher("F")),
            new NodePair(UnitTestHelper.buildBreadCrumbNode("G"), UnitTestHelper.buildNodeMatcher("G"))};
        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_PATH, currentPath, PageContext.SESSION_SCOPE);

        // check the result
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
                UnitTestHelper.buildBreadCrumbNode("A"), UnitTestHelper.buildBreadCrumbNode("E"),
                UnitTestHelper.buildBreadCrumbNode("F"), UnitTestHelper.buildBreadCrumbNode("D")};

        UnitTestHelper.assertNodeEquals("The path returned should be correct.", expected,
            pathDiscovery.getPath(currentNode, pageContext));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPath(BreadCrumbNode, PageContext)</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetPath_HasCurrentPathAccuracy7() throws Exception {
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("E");

        // build the siteMap
        Map siteMap = new HashMap();

        // the map will be A->B->C->D, A->B->C->E
        siteMap.put(UnitTestHelper.buildNodeMatcher("A"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("B")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("B"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("C"),
            Arrays.asList(
                new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("D"), UnitTestHelper.buildNodeMatcher("E")}));

        siteMap.put(UnitTestHelper.buildNodeMatcher("D"), new ArrayList());
        siteMap.put(UnitTestHelper.buildNodeMatcher("E"), new ArrayList());

        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        // build the current path, the current path is A->F->G
        NodePair[] currentPath = new NodePair[] {
            new NodePair(UnitTestHelper.buildBreadCrumbNode("A"), UnitTestHelper.buildNodeMatcher("A")),
            new NodePair(UnitTestHelper.buildBreadCrumbNode("F"), UnitTestHelper.buildNodeMatcher("F")),
            new NodePair(UnitTestHelper.buildBreadCrumbNode("G"), UnitTestHelper.buildNodeMatcher("G"))};
        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_PATH, currentPath, PageContext.SESSION_SCOPE);

        // check the result
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
                UnitTestHelper.buildBreadCrumbNode("A"), UnitTestHelper.buildBreadCrumbNode("B"),
                UnitTestHelper.buildBreadCrumbNode("C"), UnitTestHelper.buildBreadCrumbNode("E")};

        UnitTestHelper.assertNodeEquals("The path returned should be correct.", expected,
            pathDiscovery.getPath(currentNode, pageContext));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPath(BreadCrumbNode, PageContext)</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetPath_HasCurrentPathAccuracy8() throws Exception {
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("A");

        // build the siteMap
        Map siteMap = new HashMap();

        // the map will be A->B->C->A, E->D->A, the path should be E->D->A
        siteMap.put(UnitTestHelper.buildNodeMatcher("A"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("B")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("B"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("C"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("A")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("E"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("D")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("D"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("A")}));

        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        // build the current path, the current path is Z->F->G
        NodePair[] currentPath = new NodePair[] {
            new NodePair(UnitTestHelper.buildBreadCrumbNode("Z"), UnitTestHelper.buildNodeMatcher("Z")),
            new NodePair(UnitTestHelper.buildBreadCrumbNode("F"), UnitTestHelper.buildNodeMatcher("F")),
            new NodePair(UnitTestHelper.buildBreadCrumbNode("G"), UnitTestHelper.buildNodeMatcher("G"))};
        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_PATH, currentPath, PageContext.SESSION_SCOPE);

        // check the result
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
                UnitTestHelper.buildBreadCrumbNode("E"), UnitTestHelper.buildBreadCrumbNode("D"),
                UnitTestHelper.buildBreadCrumbNode("A")};

        UnitTestHelper.assertNodeEquals("The path returned should be correct.", expected,
            pathDiscovery.getPath(currentNode, pageContext));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPath(BreadCrumbNode, PageContext)</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetPath_HasCurrentPathAccuracy9() throws Exception {
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("C");

        // build the siteMap
        Map siteMap = new HashMap();

        // the map will be A->B->C->A, E->D->A, the path should be E->D->A->B->C
        siteMap.put(UnitTestHelper.buildNodeMatcher("A"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("B")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("B"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("C"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("A")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("E"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("D")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("D"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("A")}));

        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        // build the current path, the current path is A->F->G
        NodePair[] currentPath = new NodePair[] {
            new NodePair(UnitTestHelper.buildBreadCrumbNode("A"), UnitTestHelper.buildNodeMatcher("A")),
            new NodePair(UnitTestHelper.buildBreadCrumbNode("F"), UnitTestHelper.buildNodeMatcher("F")),
            new NodePair(UnitTestHelper.buildBreadCrumbNode("G"), UnitTestHelper.buildNodeMatcher("G"))};
        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_PATH, currentPath, PageContext.SESSION_SCOPE);

        // check the result
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
                UnitTestHelper.buildBreadCrumbNode("E"), UnitTestHelper.buildBreadCrumbNode("D"),
                UnitTestHelper.buildBreadCrumbNode("A"), UnitTestHelper.buildBreadCrumbNode("B"),
                UnitTestHelper.buildBreadCrumbNode("C")};
        UnitTestHelper.assertNodeEquals("The path returned should be correct.", expected,
            pathDiscovery.getPath(currentNode, pageContext));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPath(BreadCrumbNode, PageContext)</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetPath_HasCurrentPathAccuracy10() throws Exception {
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("E");

        // build the siteMap
        Map siteMap = new HashMap();

        // the map will be A->B->C->A, A->D->E, there should be no path since there is no root
        siteMap.put(UnitTestHelper.buildNodeMatcher("A"),
            Arrays.asList(
                new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("B"), UnitTestHelper.buildNodeMatcher("D")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("B"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("C"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("A")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("D"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("E")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("E"), Arrays.asList(new NodeMatcher[0]));

        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        // build the current path, the current path is A->F->G
        NodePair[] currentPath = new NodePair[] {
            new NodePair(UnitTestHelper.buildBreadCrumbNode("A"), UnitTestHelper.buildNodeMatcher("A")),
            new NodePair(UnitTestHelper.buildBreadCrumbNode("F"), UnitTestHelper.buildNodeMatcher("F")),
            new NodePair(UnitTestHelper.buildBreadCrumbNode("G"), UnitTestHelper.buildNodeMatcher("G"))};
        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_PATH, currentPath, PageContext.SESSION_SCOPE);

        // check the result
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
                UnitTestHelper.buildBreadCrumbNode("A"), UnitTestHelper.buildBreadCrumbNode("F"),
                UnitTestHelper.buildBreadCrumbNode("G"), UnitTestHelper.buildBreadCrumbNode("E")};
        UnitTestHelper.assertNodeEquals("The path returned should be correct.", expected,
            pathDiscovery.getPath(currentNode, pageContext));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPath(BreadCrumbNode, PageContext)</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetPath_NoCurrentPathAccuracy1() throws Exception {
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("E");

        // build the siteMap
        Map siteMap = new HashMap();

        // the map will be A->B->C->D, A->B->C->E, the path should be A->B->C->E
        siteMap.put(UnitTestHelper.buildNodeMatcher("A"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("B")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("B"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("C"),
            Arrays.asList(
                new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("D"), UnitTestHelper.buildNodeMatcher("E")}));

        siteMap.put(UnitTestHelper.buildNodeMatcher("D"), new ArrayList());
        siteMap.put(UnitTestHelper.buildNodeMatcher("E"), new ArrayList());

        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        // check the result
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
                UnitTestHelper.buildBreadCrumbNode("A"), UnitTestHelper.buildBreadCrumbNode("B"),
                UnitTestHelper.buildBreadCrumbNode("C"), UnitTestHelper.buildBreadCrumbNode("E")};

        UnitTestHelper.assertNodeEquals("The path returned should be correct.", expected,
            pathDiscovery.getPath(currentNode, pageContext));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPath(BreadCrumbNode, PageContext)</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetPath_NoCurrentPathAccuracy2() throws Exception {
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("D");

        // build the siteMap
        Map siteMap = new HashMap();

        // the map will be A->B->C->D, A->C->D, the path should be A->C->D
        siteMap.put(UnitTestHelper.buildNodeMatcher("A"),
            Arrays.asList(
                new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("B"), UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("B"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("C"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("D")}));

        siteMap.put(UnitTestHelper.buildNodeMatcher("D"), new ArrayList());

        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        // check the result
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
                UnitTestHelper.buildBreadCrumbNode("A"), UnitTestHelper.buildBreadCrumbNode("C"),
                UnitTestHelper.buildBreadCrumbNode("D")};

        UnitTestHelper.assertNodeEquals("The path returned should be correct.", expected,
            pathDiscovery.getPath(currentNode, pageContext));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPath(BreadCrumbNode, PageContext)</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetPath_NoCurrentPathAccuracy3() throws Exception {
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("A");

        // build the siteMap
        Map siteMap = new HashMap();

        // the map will be A->B->C->D, A->C->D, the path should be A
        siteMap.put(UnitTestHelper.buildNodeMatcher("A"),
            Arrays.asList(
                new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("B"), UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("B"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("C"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("D")}));

        siteMap.put(UnitTestHelper.buildNodeMatcher("D"), new ArrayList());

        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        // check the result
        BreadCrumbNode[] expected = new BreadCrumbNode[] {UnitTestHelper.buildBreadCrumbNode("A") };

        UnitTestHelper.assertNodeEquals("The path returned should be correct.", expected,
            pathDiscovery.getPath(currentNode, pageContext));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPath(BreadCrumbNode, PageContext)</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetPath_NoCurrentPathAccuracy4() throws Exception {
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("A");

        // build the siteMap
        Map siteMap = new HashMap();

        // the map will be A->B->C->A, E->D->A, the path should be E->D->A
        siteMap.put(UnitTestHelper.buildNodeMatcher("A"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("B")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("B"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("C"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("A")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("E"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("D")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("D"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("A")}));

        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        // check the result
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
                UnitTestHelper.buildBreadCrumbNode("E"), UnitTestHelper.buildBreadCrumbNode("D"),
                UnitTestHelper.buildBreadCrumbNode("A")};
        UnitTestHelper.assertNodeEquals("The path returned should be correct.", expected,
            pathDiscovery.getPath(currentNode, pageContext));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPath(BreadCrumbNode, PageContext)</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetPath_NoCurrentPathAccuracy5() throws Exception {
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("D");

        // build the siteMap
        Map siteMap = new HashMap();

        // the map will be A->B->C->A, E->D->A, the path should be E->D
        siteMap.put(UnitTestHelper.buildNodeMatcher("A"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("B")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("B"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("C"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("A")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("E"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("D")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("D"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("A")}));

        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        // check the result
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
                UnitTestHelper.buildBreadCrumbNode("E"), UnitTestHelper.buildBreadCrumbNode("D")};
        UnitTestHelper.assertNodeEquals("The path returned should be correct.", expected,
            pathDiscovery.getPath(currentNode, pageContext));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPath(BreadCrumbNode, PageContext)</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetPath_NoCurrentPathAccuracy6() throws Exception {
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("C");

        // build the siteMap
        Map siteMap = new HashMap();

        // the map will be A->B->C->A, E->D->A, the path should be E->D->A->B->C
        siteMap.put(UnitTestHelper.buildNodeMatcher("A"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("B")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("B"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("C"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("A")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("E"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("D")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("D"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("A")}));

        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        // check the result
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
                UnitTestHelper.buildBreadCrumbNode("E"), UnitTestHelper.buildBreadCrumbNode("D"),
                UnitTestHelper.buildBreadCrumbNode("A"), UnitTestHelper.buildBreadCrumbNode("B"),
                UnitTestHelper.buildBreadCrumbNode("C")};
        UnitTestHelper.assertNodeEquals("The path returned should be correct.", expected,
            pathDiscovery.getPath(currentNode, pageContext));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPath(BreadCrumbNode, PageContext)</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetPath_NoCurrentPathAccuracy7() throws Exception {
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("C");

        // build the siteMap
        Map siteMap = new HashMap();

        // the map will be A->B->C, A->E->C, A->D->C, the path should be A->B->C
        siteMap.put(UnitTestHelper.buildNodeMatcher("A"),
            Arrays.asList(
                new NodeMatcher[] {
                    UnitTestHelper.buildNodeMatcher("B"), UnitTestHelper.buildNodeMatcher("D"),
                    UnitTestHelper.buildNodeMatcher("E")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("B"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("D"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("E"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("C"), Arrays.asList(new NodeMatcher[0]));

        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        // check the result
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
                UnitTestHelper.buildBreadCrumbNode("A"), UnitTestHelper.buildBreadCrumbNode("B"),
                UnitTestHelper.buildBreadCrumbNode("C")};
        UnitTestHelper.assertNodeEquals("The path returned should be correct.", expected,
            pathDiscovery.getPath(currentNode, pageContext));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPath(BreadCrumbNode, PageContext)</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetPath_NoCurrentPathAccuracy8() throws Exception {
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("A");

        // build the siteMap
        Map siteMap = new HashMap();

        // the map will be A->B->C->A, A->D->E, there should be no path since there is no root
        siteMap.put(UnitTestHelper.buildNodeMatcher("A"),
            Arrays.asList(
                new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("B"), UnitTestHelper.buildNodeMatcher("D")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("B"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("C"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("A")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("D"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("E")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("E"), Arrays.asList(new NodeMatcher[0]));

        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        // check the result
        assertEquals("The path returned should be correct.", null, pathDiscovery.getPath(currentNode, pageContext));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPath(BreadCrumbNode, PageContext)</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetPath_NoCurrentPathAccuracy9() throws Exception {
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("E");

        // build the siteMap
        Map siteMap = new HashMap();

        // the map will be A->B->C->A, A->D->E, there should be no path since there is no root
        siteMap.put(UnitTestHelper.buildNodeMatcher("A"),
            Arrays.asList(
                new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("B"), UnitTestHelper.buildNodeMatcher("D")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("B"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("C"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("A")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("D"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("E")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("E"), Arrays.asList(new NodeMatcher[0]));

        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        // check the result
        assertEquals("The path returned should be correct.", null, pathDiscovery.getPath(currentNode, pageContext));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPath(BreadCrumbNode, PageContext)</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetPath_NoCurrentPathAccuracy10() throws Exception {
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("F");

        // build the siteMap
        Map siteMap = new HashMap();

        // the map will be A->B->C->A, A->D->E, there should be no path since there is no node in the site map.
        siteMap.put(UnitTestHelper.buildNodeMatcher("A"),
            Arrays.asList(
                new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("B"), UnitTestHelper.buildNodeMatcher("D")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("B"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("C"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("A")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("D"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("E")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("E"), Arrays.asList(new NodeMatcher[0]));

        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        // check the result
        assertEquals("The path returned should be correct.", null, pathDiscovery.getPath(currentNode, pageContext));
    }
}
