/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.UnitTestHelper;
import com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery.nodematchers.RegexNodeMatcher;

import org.apache.cactus.JspTestCase;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.jsp.PageContext;


/**
 * <p>
 * Tests functionality and error cases of <code>XmlPathDiscovery</code> class.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0
 */
public class XmlPathDiscoveryUnitTest extends JspTestCase {
    /** Represents the file name of the data source used for testing. */
    private static final String DATA_SOURCE = "test_files/BreadCrumbSource.xml";

    /** Represents the <code>XmlPathDiscovery</code> instance used for testing. */
    private XmlPathDiscovery pathDiscovery = null;

    /**
     * <p>
     * Sets up the test environment. The test instance is created.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    protected void setUp() throws Exception {
        pathDiscovery = new XmlPathDiscovery(DATA_SOURCE);
    }

    /**
     * <p>
     * Tests the constructor <code>XmlPathDiscovery(String)</code> with null filename, IllegalArgumentException should
     * be thrown.
     * </p>
     */
    public void testXmlPathDiscovery_StringNullFilename() {
        try {
            new XmlPathDiscovery(null);
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the constructor <code>XmlPathDiscovery(String)</code> with empty filename, IllegalArgumentException should
     * be thrown.
     * </p>
     */
    public void testXmlPathDiscovery_StringEmptyFilename() {
        try {
            new XmlPathDiscovery(" ");
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of constructor <code>XmlPathDiscovery(String)</code>.
     * </p>
     */
    public void testXmlPathDiscovery_StringAccuracy() {
        assertNotNull("The XmlPathDiscovery instance should be created.", pathDiscovery);
        assertEquals("The source should be set properly.", DATA_SOURCE,
            UnitTestHelper.getPrivateField(pathDiscovery.getClass(), pathDiscovery, "source"));
        assertTrue("The isFile should be set properly.",
            Boolean.valueOf(UnitTestHelper.getPrivateField(pathDiscovery.getClass(), pathDiscovery,
                    "isFile").toString()).booleanValue());
    }

    /**
     * <p>
     * Tests the constructor <code>XmlPathDiscovery(String, boolean)</code> with null filename,
     * IllegalArgumentException should be thrown.
     * </p>
     */
    public void testXmlPathDiscovery_StringBooleanNullFilename() {
        try {
            new XmlPathDiscovery(null, false);
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the constructor <code>XmlPathDiscovery(String)</code> with empty filename, IllegalArgumentException should
     * be thrown.
     * </p>
     */
    public void testXmlPathDiscovery_StringBooleanEmptyFilename() {
        try {
            new XmlPathDiscovery(" ", false);
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of constructor <code>XmlPathDiscovery(String)</code>.
     * </p>
     */
    public void testXmlPathDiscovery_StringBooleanAccuracy() {
        pathDiscovery = new XmlPathDiscovery(DATA_SOURCE, false);
        assertNotNull("The XmlPathDiscovery instance should be created.", pathDiscovery);
        assertEquals("The source should be set properly.", DATA_SOURCE,
            UnitTestHelper.getPrivateField(pathDiscovery.getClass(), pathDiscovery, "source"));
        assertFalse("The isFile should be set properly.",
            Boolean.valueOf(UnitTestHelper.getPrivateField(pathDiscovery.getClass(), pathDiscovery,
                    "isFile").toString()).booleanValue());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getSiteMap()</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetSiteMap_Accuracy() throws Exception {
        Map siteMap = pathDiscovery.getSiteMap();

        // check the map
        assertEquals("The siteMap should be correct.", 5, siteMap.size());

        NodeMatcher[] matchers = new NodeMatcher[5];
        matchers[0] = new RegexNodeMatcher("title root", "/title_root", "/title_root");
        matchers[1] = new RegexNodeMatcher("title Branch 1", "/urlbranch1", "/urlbranch1?.*");
        matchers[2] = new RegexNodeMatcher("title Branch 2", "/urlbranch2", "/urlbranch2");
        matchers[3] = new RegexNodeMatcher("title Branch 3", "/urlbranch3", "/urlbranch3?var=1.*");
        matchers[4] = new RegexNodeMatcher("leafTitle0", "/leafUrl", "/leafUrl?.*");

        List nodes = (List) siteMap.get(matchers[0]);
        assertNotNull("The siteMap should be correct.", nodes);
        assertEquals("The siteMap should be correct.", 3, nodes.size());
        assertEquals("The siteMap should be correct.", matchers[1], nodes.get(0));
        assertEquals("The siteMap should be correct.", matchers[2], nodes.get(1));
        assertEquals("The siteMap should be correct.", matchers[3], nodes.get(2));

        nodes = (List) siteMap.get(matchers[1]);
        assertNotNull("The siteMap should be correct.", nodes);
        assertEquals("The siteMap should be correct.", 0, nodes.size());

        nodes = (List) siteMap.get(matchers[2]);
        assertNotNull("The siteMap should be correct.", nodes);
        assertEquals("The siteMap should be correct.", 1, nodes.size());
        assertEquals("The siteMap should be correct.", matchers[4], nodes.get(0));

        nodes = (List) siteMap.get(matchers[3]);
        assertNotNull("The siteMap should be correct.", nodes);
        assertEquals("The siteMap should be correct.", 1, nodes.size());
        assertEquals("The siteMap should be correct.", matchers[4], nodes.get(0));

        nodes = (List) siteMap.get(matchers[4]);
        assertNotNull("The siteMap should be correct.", nodes);
        assertEquals("The siteMap should be correct.", 0, nodes.size());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPath(BreadCrumbNode, PageContext)</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetPath_Accuracy1() throws Exception {
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("E");

        // build the siteMap
        Map siteMap = new HashMap();

        // the map will be A->B->C->D, A->B->C->E, the path should be A->B->C->E
        siteMap.put(UnitTestHelper.buildNodeMatcher("A"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("B")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("B"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("C"),
            Arrays.asList(
                new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("D"), UnitTestHelper.buildNodeMatcher("E")}));

        siteMap.put(UnitTestHelper.buildNodeMatcher("D"), new ArrayList());
        siteMap.put(UnitTestHelper.buildNodeMatcher("E"), new ArrayList());

        this.pageContext.setAttribute(XmlPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        // check the result
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
                UnitTestHelper.buildBreadCrumbNode("A"), UnitTestHelper.buildBreadCrumbNode("B"),
                UnitTestHelper.buildBreadCrumbNode("C"), UnitTestHelper.buildBreadCrumbNode("E")};

        UnitTestHelper.assertNodeEquals("The path returned should be correct.", expected,
            pathDiscovery.getPath(currentNode, pageContext));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPath(BreadCrumbNode, PageContext)</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetPath_Accuracy2() throws Exception {
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("D");

        // build the siteMap
        Map siteMap = new HashMap();

        // the map will be A->B->C->D, A->C->D, the path should be A->C->D
        siteMap.put(UnitTestHelper.buildNodeMatcher("A"),
            Arrays.asList(
                new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("B"), UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("B"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("C"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("D")}));

        siteMap.put(UnitTestHelper.buildNodeMatcher("D"), new ArrayList());

        this.pageContext.setAttribute(XmlPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        // check the result
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
                UnitTestHelper.buildBreadCrumbNode("A"), UnitTestHelper.buildBreadCrumbNode("C"),
                UnitTestHelper.buildBreadCrumbNode("D")};

        UnitTestHelper.assertNodeEquals("The path returned should be correct.", expected,
            pathDiscovery.getPath(currentNode, pageContext));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPath(BreadCrumbNode, PageContext)</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetPath_Accuracy3() throws Exception {
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("A");

        // build the siteMap
        Map siteMap = new HashMap();

        // the map will be A->B->C->D, A->C->D, the path should be A
        siteMap.put(UnitTestHelper.buildNodeMatcher("A"),
            Arrays.asList(
                new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("B"), UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("B"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("C"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("D")}));

        siteMap.put(UnitTestHelper.buildNodeMatcher("D"), new ArrayList());

        this.pageContext.setAttribute(XmlPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        // check the result
        BreadCrumbNode[] expected = new BreadCrumbNode[] {UnitTestHelper.buildBreadCrumbNode("A") };

        UnitTestHelper.assertNodeEquals("The path returned should be correct.", expected,
            pathDiscovery.getPath(currentNode, pageContext));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPath(BreadCrumbNode, PageContext)</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetPath_Accuracy4() throws Exception {
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("A");

        // build the siteMap
        Map siteMap = new HashMap();

        // the map will be A->B->C->A, E->D->A, the path should be E->D->A
        siteMap.put(UnitTestHelper.buildNodeMatcher("A"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("B")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("B"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("C"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("A")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("E"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("D")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("D"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("A")}));

        this.pageContext.setAttribute(XmlPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        // check the result
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
                UnitTestHelper.buildBreadCrumbNode("E"), UnitTestHelper.buildBreadCrumbNode("D"),
                UnitTestHelper.buildBreadCrumbNode("A")};
        UnitTestHelper.assertNodeEquals("The path returned should be correct.", expected,
            pathDiscovery.getPath(currentNode, pageContext));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPath(BreadCrumbNode, PageContext)</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetPath_Accuracy5() throws Exception {
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("D");

        // build the siteMap
        Map siteMap = new HashMap();

        // the map will be A->B->C->A, E->D->A, the path should be E->D
        siteMap.put(UnitTestHelper.buildNodeMatcher("A"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("B")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("B"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("C"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("A")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("E"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("D")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("D"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("A")}));

        this.pageContext.setAttribute(XmlPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        // check the result
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
                UnitTestHelper.buildBreadCrumbNode("E"), UnitTestHelper.buildBreadCrumbNode("D")};
        UnitTestHelper.assertNodeEquals("The path returned should be correct.", expected,
            pathDiscovery.getPath(currentNode, pageContext));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPath(BreadCrumbNode, PageContext)</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetPath_Accuracy6() throws Exception {
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("C");

        // build the siteMap
        Map siteMap = new HashMap();

        // the map will be A->B->C->A, E->D->A, the path should be E->D->A->B->C
        siteMap.put(UnitTestHelper.buildNodeMatcher("A"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("B")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("B"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("C"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("A")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("E"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("D")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("D"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("A")}));

        this.pageContext.setAttribute(XmlPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        // check the result
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
                UnitTestHelper.buildBreadCrumbNode("E"), UnitTestHelper.buildBreadCrumbNode("D"),
                UnitTestHelper.buildBreadCrumbNode("A"), UnitTestHelper.buildBreadCrumbNode("B"),
                UnitTestHelper.buildBreadCrumbNode("C")};
        UnitTestHelper.assertNodeEquals("The path returned should be correct.", expected,
            pathDiscovery.getPath(currentNode, pageContext));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPath(BreadCrumbNode, PageContext)</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetPath_Accuracy7() throws Exception {
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("C");

        // build the siteMap
        Map siteMap = new HashMap();

        // the map will be A->B->C, A->E->C, A->D->C, the path should be A->B->C
        siteMap.put(UnitTestHelper.buildNodeMatcher("A"),
            Arrays.asList(
                new NodeMatcher[] {
                    UnitTestHelper.buildNodeMatcher("B"), UnitTestHelper.buildNodeMatcher("D"),
                    UnitTestHelper.buildNodeMatcher("E")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("B"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("D"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("E"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("C"), Arrays.asList(new NodeMatcher[0]));

        this.pageContext.setAttribute(XmlPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        // check the result
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
                UnitTestHelper.buildBreadCrumbNode("A"), UnitTestHelper.buildBreadCrumbNode("B"),
                UnitTestHelper.buildBreadCrumbNode("C")};
        UnitTestHelper.assertNodeEquals("The path returned should be correct.", expected,
            pathDiscovery.getPath(currentNode, pageContext));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPath(BreadCrumbNode, PageContext)</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetPath_Accuracy8() throws Exception {
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("A");

        // build the siteMap
        Map siteMap = new HashMap();

        // the map will be A->B->C->A, A->D->E, there should be no path since there is no root
        siteMap.put(UnitTestHelper.buildNodeMatcher("A"),
            Arrays.asList(
                new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("B"), UnitTestHelper.buildNodeMatcher("D")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("B"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("C"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("A")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("D"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("E")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("E"), Arrays.asList(new NodeMatcher[0]));

        this.pageContext.setAttribute(XmlPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        // check the result
        assertEquals("The path returned should be correct.", null, pathDiscovery.getPath(currentNode, pageContext));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPath(BreadCrumbNode, PageContext)</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetPath_Accuracy9() throws Exception {
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("E");

        // build the siteMap
        Map siteMap = new HashMap();

        // the map will be A->B->C->A, A->D->E, there should be no path since there is no root
        siteMap.put(UnitTestHelper.buildNodeMatcher("A"),
            Arrays.asList(
                new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("B"), UnitTestHelper.buildNodeMatcher("D")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("B"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("C"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("A")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("D"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("E")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("E"), Arrays.asList(new NodeMatcher[0]));

        this.pageContext.setAttribute(XmlPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        // check the result
        assertEquals("The path returned should be correct.", null, pathDiscovery.getPath(currentNode, pageContext));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPath(BreadCrumbNode, PageContext)</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testGetPath_Accuracy10() throws Exception {
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("F");

        // build the siteMap
        Map siteMap = new HashMap();

        // the map will be A->B->C->A, A->D->E, there should be no path since there is no node in the site map.
        siteMap.put(UnitTestHelper.buildNodeMatcher("A"),
            Arrays.asList(
                new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("B"), UnitTestHelper.buildNodeMatcher("D")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("B"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("C"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("A")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("D"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("E")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("E"), Arrays.asList(new NodeMatcher[0]));

        this.pageContext.setAttribute(XmlPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        // check the result
        assertEquals("The path returned should be correct.", null, pathDiscovery.getPath(currentNode, pageContext));
    }
}
